# -*- coding: utf-8 -*-

import docxtpl as dtp
import docx.shared as ds
import datetime as dt
import typing as typ
import pathlib as pth

from pacifico_devel.util.reportTemplate.src.core.Report.Report import Report
from pacifico_devel.util.lexikon.src.util.Enumerations import Language
import pacifico_devel.util.reportTemplate.src.util.CONFIG as CFG


class ReportDocx(Report):
    _OUTPUT_FORMAT = ".docx"
    _FILENAME_TEMPLATE_DEFAULT: str = "demo.docx"
    _PAGE_DIMENSIONS_DEFAULT: typ.Tuple[float, float] = (27.94, 21.59)  # CARTA, HORIZONTAL, cm, (width, height)
    _PAGE_MARGINS_DEFAULT: typ.Tuple[float, float] = (1.27, 1.27)  # ESTRECHO, cm, (width, height)

    def __init__(self,
                 pathTemplate: typ.Union[str, pth.Path] = _FILENAME_TEMPLATE_DEFAULT,
                 languageOutput: typ.Union[Language, str] = Language.Spanish,
                 languageInput: typ.Union[Language, str] = Language.Spanish,
                 date: dt.datetime = dt.datetime.now(),
                 pageDimensions: typ.Tuple[float, float] = _PAGE_DIMENSIONS_DEFAULT,
                 pageMargins: typ.Tuple[float, float] = _PAGE_MARGINS_DEFAULT):
        super().__init__(pathTemplate=pathTemplate,
                         languageOutput=languageOutput,
                         languageInput=languageInput,
                         date=date,
                         outputFormat=self._OUTPUT_FORMAT)

        self._doc: dtp.DocxTemplate = dtp.DocxTemplate(self._pathTemplate)
        self._contents: typ.Dict[str, typ.Union[str, dtp.InlineImage]] = {}

        self._widthUnit: float = pageDimensions[0] - 2 * pageMargins[0]  # cm
        self._heightUnit: float = pageDimensions[1] - 2 * pageMargins[1]  # cm

    def _setString(self,
                   tag: str,
                   value: str):
        """
        Adds string to document.

        Args:
            tag:
            value:

        Returns:

        """
        self._contents[tag] = value

    def render(self,
               pathOutput: typ.Union[str, pth.Path]) -> None:
        """
        Generates document and saves it to _PATH_OUTPUT class attribute.

        Returns:
            param pathOutput:

        """
        print("#" * 20 + "\n")
        print(f"DOCUMENT CONTENTS: {self._contents}")
        if isinstance(pathOutput, str):
            pathOutput = pth.Path(pathOutput)
        self._doc.render(self._contents)
        self._doc.save(str(pathOutput))
        print(f"SAVED TO {pathOutput}")
        self._deleteAllTempPlots()

    def setImage(self,
                 tag: str,
                 path: typ.Union[str, pth.Path],
                 width: typ.Optional[float] = None,
                 height: typ.Optional[float] = None) -> None:
        """
        Sets an image. Width is taken into account first; height is taken into account only if width is None.

        Returns:

        """
        print(f"Setting Image at {tag}...")
        if width is None:
            if height is None:
                self._contents[tag] = dtp.InlineImage(tpl=self._doc,
                                                      image_descriptor=str(path)
                                                      )
            else:
                self._contents[tag] = dtp.InlineImage(tpl=self._doc,
                                                      image_descriptor=str(path),
                                                      height=ds.Cm(height * self._heightUnit * (1 - self._EPSILON))
                                                      )
        else:
            self._contents[tag] = dtp.InlineImage(tpl=self._doc,
                                                  image_descriptor=str(path),
                                                  width=ds.Cm(width * self._widthUnit * (1 - self._EPSILON)),
                                                  )
        print(f"Setting Image at {tag}... DONE!")

    def setLogo(self,
                tag: str,
                width: typ.Optional[int] = None,
                height: typ.Optional[int] = None) -> None:
        self.setImage(tag=tag,
                      path=CFG.PATH_LOGO,
                      height=height,
                      width=width)


if __name__ == '__main__':
    from pacifico_devel.util.reportTemplate.src.util.testing import makeRandomCategoricalDf
    from pacifico_devel.util.reportTemplate.src.core.Plot.PlotBar import PlotBar
    from pacifico_devel.util.reportTemplate.src.core.Plot.PlotLine import PlotLine
    from pacifico_devel.util.reportTemplate.src.core.Plot.Table.TableDefault import TableDefault

    df = makeRandomCategoricalDf(shape=(15, 3))

    bp = PlotBar()
    bp.makePlot(df=df,
                title="bar plot",
                shape=(1 / 2, 1),
                valuesColumn="value_0")

    lp = PlotLine()
    lp.makePlot(df=df,
                title="line plot")

    tab = TableDefault()
    tab.makePlot(df=df,
                 title="table")

    text = "Este es un texto en espanol."

    rdoc = ReportDocx(languageOutput=Language.English_US)
    rdoc.setPlot(tag="bar_plot_image",
                 df=df,
                 plotKind="bar_plot",
                 title="",
                 shape=(1 / 2, 1))
    rdoc.setPlot(tag="line_plot_image",
                 df=df,
                 plotKind="line_plot",
                 title="",
                 shape=(1 / 2, 1))
    rdoc.setPlot(tag="table_image",
                 df=df,
                 plotKind="tabple_plot",
                 title="",
                 shape=(1 / 2, 1))

    rdoc.setText(tag="text",
                 value=text)

    rdoc.setText(tag="title",
                 value="Report Demo")

    rdoc.render(None)



from enum import Enum, auto
from typing import List


class Language(Enum):
    Spanish = auto()
    English_US = auto()
    French = auto()
    German = auto()
    Japanese = auto()

    @classmethod
    def fromDeeplString(cls,
                        stringRepr: str):
        for lang in cls:
            if stringRepr in {lang.deeplStringInput, lang.deeplStringOutput}:
                return lang
        msg1 = f"Invalid string representation: {stringRepr}. "
        msg2 = f"It must be one of {[lang.deeplString for lang in cls]}."
        raise ValueError(msg1 + "\n" + msg2)

    @property
    def deeplStringInput(self) -> str:
        if self == Language.English_US:
            return "EN"
        return self.deeplStringOutput

    @property
    def deeplStringOutput(self) -> str:
        if self == Language.Spanish:
            return "ES"
        elif self == Language.English_US:
            return "EN-US"  # "EN-US"
        elif self == Language.French:
            return "FR"
        elif self == Language.German:
            return "DE"
        elif self == Language.Japanese:
            return "JA"

    def thousandsSeparator(self) -> str:
        """
        SOURCE: https://en.wikipedia.org/wiki/Decimal_separator

        Returns:

        """
        spaceSeparated = {Language.French,
                          Language.English_US,
                          Language.Spanish}
        dotSeparated = {Language.German}
        commaSeparated = {Language.Japanese}
        if self in spaceSeparated:
            return " "
        elif self in dotSeparated:
            return "."
        elif self in commaSeparated:
            return ","
        raise ValueError()

    def decimalSeparator(self) -> str:
        dotSeparated = {Language.Japanese, Language.English_US}
        commaSeparated = {Language.German, Language.Spanish, Language.French}
        if self in dotSeparated:
            return "."
        elif self in commaSeparated:
            return ","
        raise ValueError()

    def dateSeparator(self) -> str:
        slashSeparated: set = {Language.Spanish,
                               Language.French}
        dashSeparated: set = {Language.English_US,
                              Language.German,
                              Language.Japanese}
        if self in slashSeparated:
            return "/"
        elif self in dashSeparated:
            return "-"

        raise ValueError()

    def dateOrder(self) -> List[str]:
        descending = {Language.English_US,
                      Language.German,
                      Language.Japanese}

        ascending = {Language.Spanish,
                     Language.French}

        if self in ascending:
            return ["%d", "%m", "%Y"]
        elif self in descending:
            return ["%Y", "%d", "%m"]

        raise NotImplementedError(f"Language not Implemented. Please implement at {__file__}.")

    def dateFormatString(self) -> str:
        return self.dateSeparator().join(self.dateOrder())

    # def longDateFormat(self) -> str:
    #     locale.setlocale(locale.LC_TIME, (self.deeplString, "UTF-8"))
    #     return "%A %#d, %B %Y"


if __name__ == '__main__':
    # import datetime as dt
    #
    # lang = Language.Spanish
    #
    # dateString = dt.date.today().strftime(lang.longDateFormat())
    # print(dateString)
    # print(Language["English_US"])
    # locale.setlocale(locale.LC_TIME, (self.deeplString, "UTF-8"))
    print(Language(Language.Spanish))



import typing as typ

HOST: str = "34.232.126.251"
PORT: str = "5432"
USER: str = "pacificoadmin"
PASSWORD: str = "PacificoR2020"

SQL_MODIFICATION_WORDS: typ.Set[str] = {"UPDATE", "DELETE", "INSERT", "CREATE", "ALTER", "CREATE", "DROP"}



def getToken() -> str:
    """
    Returns token for API.

    Returns (str): API token.

    """
    return "KmK9FqgJ5h1pSNjh1vhha6ul30vXBOBiMVlmplXf"

if __name__ == '__main__':
    import datetime as dt

    dt.timedelta()


import pandas as pd
import typing as typ

from pacifico_devel.util.reportTemplate.src.core.Plot.Table.Table import Table
from pacifico_devel.util.reportTemplate.src.core.Plot.Plot import MultiDisplay
from pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors import PacificoPlotColors
import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm


class TableDataframeImage(Table):

    def makePlot(self,
                 df: pd.DataFrame,
                 shape: typ.Tuple[int, int],
                 title: typ.Optional[str] = None,
                 backend: typ.Union[str, renm.Backend, None] = None) -> MultiDisplay:
        dfAux = self._translateDataFrame(df)
        thProps = [("background-color", PacificoPlotColors.BLUE.getCodeString()),
                   ("color", "white")]
        tdProps = []

        dfst = dfAux.style.set_table_styles([{"selector": "th", "props": thProps},
                                             {"selector": "td", "props": tdProps}])
        dfst = dfst.format("{:.2f}")
        # print(dfst.to_html())

        self._saveImageFile(dfst)
        return dfst


if __name__ == '__main__':
    from pacifico_devel.util.reportTemplate.src.util.testing import makeRandomCategoricalDf

    df = makeRandomCategoricalDf(shape=(20, 3))

    tdi = TableDataframeImage()
    tdi.makePlot(df=df,
                 title="",
                 backend="bokeh",
                 shape=(1, 1 / 2))



"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import time
import boto3
import json
import os
from ...util.cfg import Configuration
from ...util.aws import S3, SSH, Enumerations


def getAllInstancesObjects(awsAccessKey, awsSecretKey):
    ec2 = boto3.resource('ec2', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey, region_name='us-west-2')
    return ec2.instances.all()

def getAllInstancesSnapshot(awsAccessKey, awsSecretKey):
    instancesObjectsList = getAllInstancesObjects(awsAccessKey, awsSecretKey)
    instances = {}
    for instance in instancesObjectsList:
        data = {'Platform': instance.platform, 'Type': instance.instance_type, 'Public IPv4': instance.public_ip_address, 'AMI': instance.image.id, 'State': instance.state}
        instances.update({instance.id: data})
    return instances

def getInstance(instanceId, awsAccessKey, awsSecretKey):
    ec2 = boto3.resource('ec2', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey, region_name='us-west-2')
    return ec2.Instance(id=instanceId)

def getInstanceState(instanceId, awsAccessKey, awsSecretKey):
    instance = getInstance(instanceId, awsAccessKey, awsSecretKey)
    return instance.state  # https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_InstanceState.html

def getInstanceStateCode(instanceId, awsAccessKey, awsSecretKey):
    state = getInstanceState(instanceId, awsAccessKey, awsSecretKey)
    return state['Code']

def isInstanceRunning(instanceId, awsAccessKey, awsSecretKey):
    code = getInstanceStateCode(instanceId, awsAccessKey, awsSecretKey)
    if code == 16:
        return True
    else:
        return False

def isInstanceStopped(instanceId, awsAccessKey, awsSecretKey):
    code = getInstanceStateCode(instanceId, awsAccessKey, awsSecretKey)
    if code == 80:
        return True
    else:
        return False

def getInstancePublicIPAddress(instanceId, awsAccessKey, awsSecretKey):
    instance = getInstance(instanceId, awsAccessKey, awsSecretKey)
    return instance.public_ip_address

def startInstace(instanceId, awsAccessKey, awsSecretKey):
    instance = getInstance(instanceId, awsAccessKey, awsSecretKey)
    response = instance.start()
    return response

def stopInstace(instanceId, awsAccessKey, awsSecretKey):
    instance = getInstance(instanceId, awsAccessKey, awsSecretKey)
    response = instance.stop()
    return response

def rebootInstace(instanceId, awsAccessKey, awsSecretKey):
    instance = getInstance(instanceId, awsAccessKey, awsSecretKey)
    response = instance.reboot()
    return response

def getAllRunningInstances(awsAccessKey, awsSecretKey):
    running = []
    instances = getAllInstancesObjects(awsAccessKey, awsSecretKey)
    for instance in instances:
        if instance.state['Name'] == 'running':
            running.append(instance)
    return running

def getAllStopedInstances(awsAccessKey, awsSecretKey):
    running = []
    instances = getAllInstancesObjects(awsAccessKey, awsSecretKey)
    for instance in instances:
        if instance.state['Name'] == 'stopped':
            running.append(instance)
    return running

def __getFileName(dateNow, folder):
    """This function creates a filename based on the current time (timestamp)."""
    path = '{}{}/'.format(__getRoot(), folder)
    filename = dateNow.strftime("%d.%m.%Y-%H.%M.%S.%f") + '.txt'
    return '{}{}'.format(path, filename)

def __getRoot():
    return '/bfi/'

def getFileNameOutput(dateNow):
    return __getFileName(dateNow, 'output')

def getFileNameInput(dateNow):
    return __getFileName(dateNow, 'input')

def getFileNameError():
    return '{}output/error.txt'.format(__getRoot())

def getCommand(message, dateNow, instanceId, pemFilePath, awsAccessKey, awsSecretKey):
    filenameOutput = getFileNameOutput(dateNow)  # '/bfi/output/fileOpen.txt'
    filenameInput = ''
    if message.getTask() == '':
        command = 'error'
    elif message.getTask() == 'tickers':
        command = '-tickers {}'.format(filenameOutput)
    elif message.getTask() == 'job':
        filenameInput = getFileNameInput(dateNow)
        filenameError = getFileNameError()
        payload = S3.downloadFromBucket(filepathBucket=message.getFilenameInput(), filepathLocal='/{}'.format(message.getFilenameInput()), read=True)
        __saveFile(payload, filenameInput, instanceId, pemFilePath, awsAccessKey, awsSecretKey)
        command = '-job {} {} {}'.format(filenameInput, filenameError, filenameOutput)
    else:
        command = '-{}'.format(message.getTask())
        args = message.getArguments()
        for arg in args:
            command = '{} {}'.format(command, arg)
    if command != 'error':
        command = './bfi_api ' + command  # Testing: './bfi_test '
    return command, filenameOutput, filenameInput

def getResponse(status, filenameOutput, instanceId, pemFilePath, awsAccessKey, awsSecretKey):
    if status != 0:
        return 'error'
    keysString = __loadFile(filenameOutput, instanceId, pemFilePath, awsAccessKey, awsSecretKey) # linea es un input de la "lista"
    keys = keysString.splitlines()
    content = {}
    counter = -1
    for key in keys:
        if key[:5] == __getRoot():
            value = __loadFile(key, instanceId, pemFilePath, awsAccessKey, awsSecretKey)
        else:
            value = key
            counter += 1
        if value == key and value != '':
            content.update({'message_{}'.format(counter): value})
        elif value != '':
            content.update({key: value})
    return json.dumps(content)

def __loadFile(filename, instanceId, pemFilePath, awsAccessKey, awsSecretKey):
    if filename[:5] == '/bfi/':
        file = SSH.readFileFromInstance(filename, instanceId, pemFilePath, awsAccessKey, awsSecretKey)
    else:
        file = ''
    return file

def __saveFile(data, filename, instanceId, pemFilePath, awsAccessKey, awsSecretKey):
    if filename[:5] == '/bfi/':
        SSH.writeFileStringToInstance(data, filename, instanceId, pemFilePath, awsAccessKey, awsSecretKey)

def __deleteFile(filename, instanceId, pemFilePath, awsAccessKey, awsSecretKey):
    if filename[:5] == '/bfi/':
        SSH.deleteFileFromInstance(filename, instanceId, pemFilePath, awsAccessKey, awsSecretKey)

def getInstanceId(message, dateNow):
    computerStatic2InstanceId = Configuration.get('COMPUTER_STATIC_2_INSTANCE_ID')
    computer2InstanceId = Configuration.get('COMPUTER_2_INSTANCE_ID')
    difficulty = message.getDifficulty()
    if difficulty == Enumerations.Difficulty.Difficulty_Low:
        return computerStatic2InstanceId
    elif difficulty == Enumerations.Difficulty.Difficulty_Standard:
        return computerStatic2InstanceId
    elif difficulty == Enumerations.Difficulty.Difficulty_Static:
        return computerStatic2InstanceId
    elif difficulty == Enumerations.Difficulty.Difficulty_High:
        return computer2InstanceId
    else:
        return computerStatic2InstanceId

def downloadPemFile(instanceId, awsAccessKey='', awsSecretKey='', tmp=False):
    filepathLocal = 'computer_key.pem'
    filepathBucket = 'keys/{}'.format(filepathLocal)
    if tmp:
        filepathLocal = '{}{}'.format('/tmp/', filepathLocal)
    bucket = 'bfi-cfg'
    if os.path.isfile(filepathLocal):
        os.remove(filepathLocal)
    S3.downloadFromBucket(filepathBucket, filepathLocal, bucket, awsAccessKey, awsSecretKey)
    if not os.path.isfile(filepathLocal):
        time.sleep(3)
        S3.downloadFromBucket(filepathBucket, filepathLocal, bucket, awsAccessKey, awsSecretKey)
        time.sleep(3)
    return filepathLocal

if __name__ == '__main__':
    awsAccessKey = Configuration.get('AWS_ACCESS_KEY')
    awsSecretKey = Configuration.get('AWS_SECRET_KEY')
    instanceId = Configuration.get('COMPUTER_STATIC_2_INSTANCE_ID')
    print(instanceId)
    running = isInstanceRunning(instanceId, awsAccessKey, awsSecretKey)
    print(running)
    stopped = isInstanceStopped(instanceId, awsAccessKey, awsSecretKey)
    print(stopped)
    pemFilePath = downloadPemFile(instanceId, awsAccessKey, awsSecretKey, tmp=False)
    print(pemFilePath)
    response = getResponse(0, '/bfi/output/fileOpen.txt', instanceId, pemFilePath, awsAccessKey, awsSecretKey)
    print(response)


"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import datetime
import boto3
import os
import requests
import time
from ..cfg import Configuration
from .. import Dates

def getS3Client(awsAccessKey='', awsSecretKey=''):
    if (awsAccessKey != '' and awsSecretKey != ''):
        s3 = boto3.client('s3', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey, region_name='us-west-2')
    else:
        awsAccessKey = Configuration.get('AWS_ACCESS_KEY')
        awsSecretKey = Configuration.get('AWS_SECRET_KEY')
        s3 = boto3.client('s3', aws_access_key_id=awsAccessKey, aws_secret_access_key=awsSecretKey, region_name='us-west-2')
    return s3

def getUrl(fileName, bucketName='bfi-media', timeout_s=86400, awsAccessKey='', awsSecretKey=''):
    client = getS3Client(awsAccessKey, awsSecretKey)
    response = client.generate_presigned_url('get_object', Params={'Bucket': bucketName, 'Key': fileName}, ExpiresIn=timeout_s)
    return response

def writeToBucket(stringData, filename, bucketName='bfi-media', awsAccessKey='', awsSecretKey='', bucketFilename=''):
    with open(filename, 'w') as outFile:
        outFile.write(stringData)
    writeFileToBucket(filename, bucketName, awsAccessKey, awsSecretKey, bucketFilename)

def writeFileToBucket(filename, bucketName='bfi-media', awsAccessKey='', awsSecretKey='', bucketFilename='', eraseLocalFile: bool=True):
    if bucketFilename == '':
        bucketFilename = filename
    client = getS3Client(awsAccessKey, awsSecretKey)
    client.upload_file(filename, bucketName, bucketFilename)
    if eraseLocalFile:
        removeLocalFile(filename)

def downloadFromBucket(filepathBucket, filepathLocal='', bucketName='bfi-media', awsAccessKey='', awsSecretKey='', read=False):
    client = getS3Client(awsAccessKey, awsSecretKey)
    if filepathLocal == '':
        filepathLocal = filepathBucket.split('/')[-1]
    client.download_file(bucketName, filepathBucket, filepathLocal)
    time.sleep(0.1)
    if not os.path.isfile(filepathLocal):
        return downloadFromBucket(filepathBucket, filepathLocal, bucketName, awsAccessKey, awsSecretKey, read)
    else:
        if read:
            with open(filepathLocal) as f:
                inputString = f.read()
            return inputString

def getFileFromBucketAsString(filepathBucket, bucketName='bfi-media', awsAccessKey='', awsSecretKey=''):
    client = getS3Client(awsAccessKey, awsSecretKey)
    data = client.get_object(Bucket=bucketName, Key=filepathBucket)['Body'].read()
    return data.decode("utf-8-sig")

def removeLocalFile(filename):
    if os.path.isfile(filename):
        os.remove(filename)

def __readCSVFromString(csvString, separator=';'):
    csvAsMatrix = []
    lines = csvString.splitlines()
    for line in lines:
        values = line.split(separator)
        csvAsMatrix.append(values)
    return csvAsMatrix

def getCSVFromBucketAsListOfLists(filepathBucket, bucketName='bfi-media', awsAccessKey='', awsSecretKey='', separator=';'):
    stringData = getFileFromBucketAsString(filepathBucket, bucketName, awsAccessKey, awsSecretKey)
    listOflists = __readCSVFromString(stringData, separator)
    return listOflists

def writeToUrl(url, response):  # TEST
    filename = str(Dates.getDateTimeTimestamp(datetime.datetime.now())) + '.txt'
    with open(filename, 'w') as outFile:
        outFile.write(response)
    response = requests.post(url['url'], data=url['fields'], files={'file': open(filename, 'rb')})
    print(str(response.content), str(response.text))
    removeLocalFile(filename)


if __name__ == '__main__':
    awsAccessKey = Configuration.get('AWS_ACCESS_KEY')
    awsSecretKey = Configuration.get('AWS_SECRET_KEY')
    filenpath = 'keys/computer_key.pem'
    bucketName = 'bfi-cfg'
    #downloadFromBucket(filenpath, '', bucketName, awsAccessKey, awsSecretKey)
    print(getFileFromBucketAsString(filenpath, bucketName, awsAccessKey, awsSecretKey))


# -*- coding: utf-8 -*-

import abc
import datetime as dt
from abc import ABCMeta, abstractmethod
import pandas as pd
import pathlib as pth
import typing as typ
import re

from pacifico_devel import Message
from pacifico_devel.api.src.util import Enumerations, Variant

from pacifico_devel.util.reportTemplate.src.core.Style.Style import Style
from pacifico_devel.util.reportTemplate.src.core.Plot.Plot import Plot
import pacifico_devel.util.reportTemplate.src.io.PlotFactory as pltf
import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm
import pacifico_devel.util.reportTemplate.src.util.Report.DateFormatterEn as dtf

from pacifico_devel.util.lexikon.src.util.Enumerations import Language
from pacifico_devel.util.lexikon.src.core.Translator import Translator

from pacifico_devel.util.aws import S3
from pacifico_devel.util.cfg import Configuration as cfg

multiDate = typ.Union[dt.date, dt.datetime]


class Report(metaclass=ABCMeta):
    """
    Puts it all together? (format and style, analogue to HTML and CSS.)
    """

    _BASE_PLOT_RESOLUTION: int = 1000
    _EPSILON: float = 2 / 100

    def __init__(self,
                 pathTemplate: typ.Union[pth.Path, str],
                 languageOutput: typ.Union[Language, str] = Language.Spanish,
                 languageInput: typ.Union[Language, str] = Language.Spanish,
                 date: dt.datetime = dt.datetime.now(),
                 outputFormat: typ.Optional[str] = None,
                 plotBackend: renm.Backend = renm.Backend.ALTAIR):

        self._plotBackend = plotBackend
        self._pathTemplate: pth.Path = pth.Path(pathTemplate)
        self._checkPathTemplate()

        if outputFormat is None:
            self._outputFormat: str = pathTemplate.suffix
        else:
            self._outputFormat: str = outputFormat

        if isinstance(languageOutput, str):
            self._languageOutput = Language.fromDeeplString(languageOutput)
        else:
            self._languageOutput = languageOutput

        if isinstance(languageInput, str):
            self._languageInput = Language.fromDeeplString(languageInput)
        else:
            self._languageInput = languageInput

        if not self._pathTemplate.suffix == self._outputFormat:
            raise ValueError(f"File {self._pathTemplate} must be a {self._outputFormat} document.")

        self._date = date
        self._tempPlots: typ.Dict[str, Plot] = {}

    @property
    def languageOutput(self) -> Language:
        return self._languageOutput

    @property
    def languageInput(self) -> Language:
        return self._languageInput

    @abc.abstractmethod
    def render(self,
               pathOutput: typ.Union[str, pth.Path]) -> None:
        """
        Generates document and saves it locally.

        Args:
            pathOutput:

        Returns:

        """
        pass

    def export(self,
               applicationName: str) -> typ.List[Message.Report]:
        """
        Make a file from abstract reportTemplate and save it to a S3 stated in the filename.

        Args:
           applicationName:

        Returns (str): URL of document.

        """
        self.render(pathOutput=self._tempOutputPath)
        url = self._uploadAndGetUrl(filepathLocal=self._tempOutputPath)
        dateEffective = dt.datetime.now()
        message = self._getMessage(url=url,
                                   applicationName=applicationName,
                                   dateEffective=dateEffective)
        return [message]

    def _deleteAllTempPlots(self) -> None:
        """
        deletes all temporary plot images

        Returns:

        """
        [plot.deleteImageFile()
         for plot in self._tempPlots.values()]

    def _getMessage(self,
                    url: str,
                    applicationName: str,
                    dateEffective: dt.datetime) -> Message.Report:
        """
        Returns information in API format (Message.Report).

        Args:
            url:
            applicationName:
            dateEffective:

        Returns:

        """
        # document, chapter, section, subsection, paragraph = self.splitApplicationName(applicationName=applicationName)
        keys: typ.List[str] = ["document", "chapter", "section", "subsection", "paragraph"]
        values: typ.List[str] = self._splitApplicationName(applicationName=applicationName)

        if not 1 <= len(values) <= 4:
            msg1 = "Expecting name to be split into list of between 1 and 4 elements, "
            msg2 = f"but returned: {values}"
            raise ValueError(msg1 + msg2)

        values.append(self._outputFormat)  # put format last
        argDict: typ.Dict[str, typ.Union[str, dt.datetime, Variant.Variant]] = {k: v
                                                                                for k, v in zip(keys, values)}
        argDict["item"] = "report"
        argDict["dateEffective"] = dateEffective
        variant: Variant.Variant = Variant.Variant(value=url,
                                                   valueType=Enumerations.ValueType.ValueType_Url)
        print(f"URL: {url}.")
        argDict["variant"] = variant

        # message: Message.Report = Message.Report(document=document,
        #                                          chapter=chapter,
        #                                          item="report",
        #                                          variant=variant,
        #                                          dateEffective=dateEffective,
        #                                          section=section,
        #                                          subsection=subsection,
        #                                          paragraph=paragraph)
        message: Message.Report = Message.Report(**argDict)
        return message

    @abstractmethod
    def _setString(self,
                   tag,
                   value: str) -> None:
        """
        Adds string to document.

        Args:
            tag:
            value:

        Returns:

        """
        pass

    @abstractmethod
    def setImage(self,
                 tag: str,
                 path: typ.Union[str, pth.Path],
                 width: typ.Optional[float] = None,
                 height: typ.Optional[float] = None) -> None:
        """
        Sets an image.

        Returns:

        """

        pass

    def setString(self,
                  tag: str,
                  value: str,
                  translate: bool = False) -> None:  # could be enumeration or beautifulSoup:
        """
        For bond mnemo (for example).

        Args:
            value:
            tag:
            translate:

        Returns:

        """
        valueString = value
        if translate:
            print("\n" + "#" * 20)
            print("TRANSLATING...")
            msg1 = f"TEXT TO TRANSLATE FROM {self._languageInput.deeplStringInput} to {self._languageOutput.deeplStringOutput}:"
            msg2 = f"\n\t{valueString}"
            print(msg1 + msg2)
            valueString = Translator.translate(text=valueString,
                                               languageInput=self.languageInput,
                                               languageOutput=self.languageOutput)
            print(f"TRANSLATION : \n\t{valueString}")
            print("TRANSLATING... DONE!")
            print("#" * 20 + "\n")
        self._setString(value=valueString,
                        tag=tag)

    def setText(self,
                tag: str,
                value: str):
        """
        For Commentary (for example)

        Args:
            value:
            tag:

        Returns:

        """
        return self.setString(value=value,
                              tag=tag,
                              translate=True)

    def setInteger(self,
                   tag: str,
                   value: int) -> None:
        self.setString(value=Style.convertIntToString(value,
                                                      self.languageOutput),
                       tag=tag)

    def setFloat(self,
                 tag: str,
                 value: float,
                 numberDecimals: int,
                 isPercentage: bool = False) -> None:
        """
        Same as above.

        Args:
            value:
            numberDecimals:
            tag:
            isPercentage:

        Returns:

        """
        valueString = Style.convertFloatToString(value=value,
                                                 numberDecimals=numberDecimals,
                                                 language=self.languageOutput)

        self.setString(value=valueString,
                       tag=tag)

    def setLongDate(self,
                    tag: str,
                    value: typ.Optional[multiDate] = None,
                    ) -> None:

        if value is None:
            longDateStringEn = dtf.DateFormatterEn.getLongDateStringEn(self._date)
        else:
            longDateStringEn = dtf.DateFormatterEn.getLongDateStringEn(value)

        longDateString = Translator.translate(text=longDateStringEn,
                                              languageInput=Language.English_US,
                                              languageOutput=self.languageOutput)
        self.setString(value=longDateString,
                       tag=tag,
                       translate=False)

    def setDateTime(self,
                    value: dt.datetime,
                    tag: str) -> None:
        valueString = Style.convertDateTimeToString(value=value,
                                                    language=self.languageOutput)
        self.setString(value=valueString,
                       tag=tag)

    def setDate(self,
                value: dt.date,
                tag: str) -> None:
        """


        Args:
            value:
            tag:

        Returns:

        """
        valueString = Style.convertDateToString(value,
                                                language=self.languageOutput)

        self.setString(value=valueString,
                       tag=tag)

    def setDateMonth(self,
                     value: dt.date,
                     tag: str) -> None:
        """
        YYYY/MM

        Args:
            value:
            tag:

        Returns:

        """
        valueString = Style.convertDateMonthToString(value,
                                                     language=self.languageOutput)

        self.setString(value=valueString,
                       tag=tag)

    def setDateQuarter(self,
                       value: dt.date,
                       tag: str) -> None:
        """
        YYYY

        Args:
            value:
            tag:

        Returns:

        """
        valueString = Style.convertDateQuarterToString(value,
                                                       language=self.languageOutput)

        self.setString(value=valueString,
                       tag=tag)

    def setDateYear(self,
                    value: dt.date,
                    tag: str) -> None:
        """
        YYYY

        Args:
            value:
            tag:

        Returns:

        """
        valueString = Style.convertDateYearToString(value)

        self.setString(value=valueString,
                       tag=tag)

    def setPlot(self,
                tag: str,
                df: pd.DataFrame,
                plotKind: typ.Union[renm.PlotKind, str],
                title: str,
                shape: typ.Tuple[float, float],
                backend: typ.Union[renm.Backend, str, None] = None):  # width, height

        if backend is None:
            backend = self._plotBackend

        if isinstance(plotKind, str):
            plotKind = renm.PlotKind[plotKind.upper()]

        p = pltf.PlotFactory.getPlot(plotKind=plotKind,
                                     languageInput=self._languageInput,
                                     languageOutput=self._languageOutput)

        if title:
            titleTranslated = Translator.translate(title,
                                                   languageInput=self._languageInput,
                                                   languageOutput=self._languageOutput)
        else:
            titleTranslated = title

        p.makePlot(df=df,
                   title=titleTranslated,
                   shape=tuple(int(dim * self._BASE_PLOT_RESOLUTION)
                               for dim in shape
                               ),
                   backend=backend)
        self.setImage(tag=tag,
                      path=p.pathImage,
                      width=shape[0],
                      height=shape[1])
        self._tempPlots[tag] = p

    def setLinePlot(self,
                    tag: str,
                    df: pd.DataFrame,
                    title: str,
                    shape: typ.Tuple[float, float]) -> None:

        p = pltf.PlotFactory.getPlot(plotKind=renm.PlotKind.LINE_PLOT,
                                     languageInput=self._languageInput,
                                     languageOutput=self._languageOutput)

        if title:
            titleTranslated = Translator.translate(title,
                                                   languageInput=self._languageInput,
                                                   languageOutput=self._languageOutput)
        else:
            titleTranslated = title

        p.makePlot(df=df,
                   title=titleTranslated,
                   shape=tuple(int(dim * self._BASE_PLOT_RESOLUTION)
                               for dim in shape
                               ),
                   backend=self._plotBackend)
        self.setImage(tag=tag,
                      path=p.pathImage,
                      width=shape[0],
                      height=shape[1])
        self._tempPlots[tag] = p

    def setBarplot(self,
                   tag: str,
                   df: pd.DataFrame,
                   title: str,
                   shape: typ.Tuple[float, float]) -> None:
        p = pltf.PlotFactory.getPlot(plotKind=renm.PlotKind.BAR_PLOT,
                                     languageInput=self._languageInput,
                                     languageOutput=self._languageOutput)

        if title:
            titleTranslated = Translator.translate(title,
                                                   languageInput=self._languageInput,
                                                   languageOutput=self._languageOutput)
        else:
            titleTranslated = title

        p.makePlot(df=df,
                   title=titleTranslated,
                   shape=tuple(int(dim * self._BASE_PLOT_RESOLUTION)
                               for dim in shape
                               ),
                   backend=self._plotBackend)
        self.setImage(tag=tag,
                      path=p.pathImage,
                      width=shape[0],
                      height=shape[1])
        self._tempPlots[tag] = p

    def setTable(self,
                 df: pd.DataFrame,
                 tag: str,
                 shape: typ.Tuple[int, int] = (10, 10)):
        """
        PLAN: df (Done through Style) -> HTML -> IMAGE -> Paste in reportTemplate.

        Args:
            df:
            tag:
            shape:

        Returns:

        """
        p = pltf.PlotFactory.getPlot(plotKind=renm.PlotKind.TABLE_PLOT,
                                     languageInput=self._languageInput,
                                     languageOutput=self._languageOutput)

        p.makePlot(df=df,
                   shape=tuple(int(dim * self._BASE_PLOT_RESOLUTION)
                               for dim in shape
                               ),
                   backend=self._plotBackend)
        self.setImage(tag=tag,
                      path=p.pathImage,
                      width=shape[0],
                      height=shape[1])
        self._tempPlots[tag] = p

    @property
    def _tempOutputPath(self) -> pth.Path:
        fileName: str = self._pathTemplate.stem + "_temp" + self._outputFormat
        return self._pathTemplate.parent / fileName

    def _checkPathTemplate(self) -> None:
        """
        Assures the input template's path exists.

        """
        if not self._pathTemplate.is_file():
            msg = f"No such template path: {self._pathTemplate}. Please initialize with a valid template path."
            raise FileNotFoundError(msg)

    def _checkOuputFormat(self) -> None:
        """
        Assures input template's format is the output's file format.

        """
        if not self._pathTemplate.suffix == self._outputFormat:
            raise ValueError(f"File {self._pathTemplate} must be a {self._outputFormat} document.")

    def _uploadAndGetUrl(self, filepathLocal: typ.Union[str, pth.Path]) -> str:

        """
        Uploads file to S3 and returns URL.

        Args:
            filepathLocal:

        Returns:

        """

        if isinstance(filepathLocal, str):
            filepathLocal = pth.Path(filepathLocal)

        filepathBucket: pth.Path = self._getFilePathUpload(filePathLocal=filepathLocal)
        bucketName = "media-pacifico"
        awsAccessKey = cfg.get("AWS_ACCESS_KEY_PACIFICO")
        awsSecretKey = cfg.get("AWS_SECRET_KEY_PACIFICO")
        S3.writeFileToBucket(filepathLocal.as_posix(),
                             bucketName,
                             awsAccessKey,
                             awsSecretKey,
                             filepathBucket.as_posix())  # also erases file
        url = S3.getUrl(filepathBucket.as_posix(),
                        bucketName,
                        awsAccessKey=awsAccessKey,
                        awsSecretKey=awsSecretKey)
        return url

    @staticmethod
    def _splitApplicationName(applicationName: str) -> typ.List[str]:
        splitName: typ.List[str] = re.findall("[A-Z][a-z]+",
                                              applicationName)  # NEEDS TO INCLUDE CAPITAL LETTERS FOLLOWING
        splitName = [s for s in splitName if s != "Application"]
        if len(splitName) == 0:
            splitName = ["Report"]
        if len(splitName) >= 5:
            sFinal = " ".join(splitName[4:])
            splitName = splitName[:4]
            splitName[4] = sFinal
        return splitName

    @staticmethod
    def _getFilePathUpload(filePathLocal: pth.Path) -> pth.Path:
        stringTime: str = dt.datetime.now().strftime("%Y.%m.%d_%H.%M.%S.%f")
        fileNameUpload = filePathLocal.stem + "_" + stringTime + filePathLocal.suffix
        return pth.Path(fileNameUpload)


if __name__ == '__main__':
    # print(__file__)
    # print(pth.Path(pth.Path(__file__)))
    # test = "ApplicationStuffABTesting"
    # print(Report.splitApplicationName(test))
    filepathBucket = "template_temp_2022.04.22_17.17.36.906014.docx"
    filepathLocal = "template_temp_2022.04.22_17.17.36.906014.docx"
    bucketName = "media-pacifico"
    awsAccessKey = cfg.get("AWS_ACCESS_KEY_PACIFICO")
    awsSecretKey = cfg.get("AWS_SECRET_KEY_PACIFICO")
    # S3.writeFileToBucket(filepathLocal.as_posix(),
    #                      bucketName,
    #                      awsAccessKey,
    #                      awsSecretKey,
    #                      filepathBucket.as_posix())  # also erases file
    url = S3.getUrl(filepathBucket,
                    bucketName,
                    awsAccessKey=awsAccessKey,
                    awsSecretKey=awsSecretKey)
    print(url)



import enum
import typing as typ


class DataBase(enum.Enum):
    External_Data = enum.auto()
    External_Data_Raw = enum.auto()
    Inflacion = enum.auto()
    Tasas = enum.auto()
    mx = enum.auto()
    mx2 = enum.auto()
    postgres = enum.auto()
    webmonitor = enum.auto()

    def __str__(self) -> str:
        return self.name

    @classmethod
    def options(cls) -> typ.List[str]:
        return [db.name for db in cls]


if __name__ == '__main__':
    print(DataBase.options())

    # print(DataBase.External_Data)



"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import json
import datetime
from pacifico.util import Variant
from pacifico.util import Enumerations, Dates
from pacifico.core.Service.Field import Field


class FieldReport(Field.Field):
    def __init__(self, datePublication, fixing, dateEffective, dateTenor, document, item, chapter, section, subsection, paragraph, variant, other=''):
        self.datePublication = datePublication
        self.fixing = fixing
        self.setDateEffective(dateEffective)
        self.setDateTenor(dateTenor)
        documentIsEmpty = document == ''
        itemIsEmpty = item == ''
        if documentIsEmpty and itemIsEmpty:
            if documentIsEmpty:
                raise TypeError("The document must not be an empty string.")
            elif itemIsEmpty:
                raise TypeError("The item must not be an empty string.")
        if chapter == '':
            section = ''
        if section == '':
            subsection = ''
        if subsection == '':
            paragraph = ''
        self.document = document
        self.chapter = chapter
        self.section = section
        self.subsection = subsection
        self.paragraph = paragraph
        self.item = item
        self.variant = variant
        self.other = other

    # fixing
    def _get_fixing(self):
        return self.__fixing
    def _set_fixing(self, value):
        if not isinstance(value, Enumerations.Fixing):
            raise TypeError("The fixing must be set to a class 'Enumerations.Fixing'.")
        self.__fixing = value
    fixing = property(_get_fixing, _set_fixing)

    # document
    def _get_document(self):
        return self.__document
    def _set_document(self, value):
        if not isinstance(value, str):
            raise TypeError("The document must be set to a string.")
        self.__document = value
    document = property(_get_document, _set_document)

    # item
    def _get_item(self):
        return self.__item
    def _set_item(self, value):
        if not isinstance(value, str):
            raise TypeError("The item must be set to a string.")
        self.__item = value
    item = property(_get_item, _set_item)

    # chapter
    def _get_chapter(self):
        return self.__chapter
    def _set_chapter(self, value):
        if not isinstance(value, str):
            raise TypeError("The chapter must be set to a string.")
        self.__chapter = value
    chapter = property(_get_chapter, _set_chapter)

    # section
    def _get_section(self):
        return self.__section
    def _set_section(self, value):
        if not isinstance(value, str):
            raise TypeError("The section must be set to a string.")
        self.__section = value
    section = property(_get_section, _set_section)

    # subsection
    def _get_subsection(self):
        return self.__subsection
    def _set_subsection(self, value):
        if not isinstance(value, str):
            raise TypeError("The subsection must be set to a string.")
        self.__subsection = value
    subsection = property(_get_subsection, _set_subsection)

    # paragraph
    def _get_paragraph(self):
        return self.__paragraph
    def _set_paragraph(self, value):
        if not isinstance(value, str):
            raise TypeError("The paragraph must be set to a string.")
        self.__paragraph = value
    paragraph = property(_get_paragraph, _set_paragraph)

    # variant
    def _get_variant(self):
        return self.__variant
    def _set_variant(self, value):
        if not isinstance(value, Variant.Variant):
            raise TypeError("The variant must be set to a class 'Variant.Variant'.")
        self.__variant = value
    variant = property(_get_variant, _set_variant)

    # other
    def _get_other(self):
        return self.__other
    def _set_other(self, value):
        if not isinstance(value, str):
            raise TypeError("The other must be set to a string.")
        self.__other = value
    other = property(_get_other, _set_other)

    def getDatePublication(self):
        return self.datePublication

    def getFixing(self):
        return self.fixing

    def getDateEffective(self):
        return self.dateEffective

    def getDateTenor(self):
        return self.dateTenor

    def getDocument(self):
        return self.document

    def getItem(self):
        return self.item

    def getChapter(self):
        return self.chapter

    def getSection(self):
        return self.section

    def getSubsection(self):
        return self.subsection

    def getParagraph(self):
        return self.paragraph

    def getVariant(self):
        return self.variant

    def getOther(self):
        return self.other

    def setDateEffective(self, dateEffective):
        if isinstance(dateEffective, (datetime.datetime, datetime.date)):
            self.dateEffective = dateEffective
        else:
            if dateEffective != '':
                self.dateEffective = Dates.dateTimeFromString(dateEffective)
            else:
                if isinstance(self.getDatePublication(), str):
                    if Dates.isDateTimeFromDateTimeOrDateString(self.getDatePublication()):
                        self.dateEffective = Dates.dateTimeFromString(self.getDatePublication())
                    else:
                        datePublication = Dates.dateFromString(self.getDatePublication())
                        hour, minute, second = Enumerations.Fixing.getTime(self.getFixing())
                        self.dateEffective = Dates.createDateTimeFromValues(datePublication.year, datePublication.month, datePublication.day, hour, minute, second)
                else:
                    hour, minute, second = Enumerations.Fixing.getTime(self.getFixing())
                    self.dateEffective = Dates.createDateTimeFromValues(self.getDatePublication().year, self.getDatePublication().month, self.getDatePublication().day, hour, minute, second)

    def setDateTenor(self, dateTenor):
        if isinstance(dateTenor, (datetime.datetime, datetime.date)):
            self.dateTenor = dateTenor
        else:
            if dateTenor != '':
                self.dateTenor = Dates.dateTimeFromString(dateTenor)
            else:
                self.dateTenor = self.getDateEffective()

    def getBase(self):
        for base in self.__class__.__bases__:
            return base.__name__

    @staticmethod
    def __serializeAndClean(dictionary):
        delete = []
        for key, value in zip(dictionary.keys(), dictionary.values()):
            #if key in ['fixing', 'datePublication']:
            #    delete.append(key)
            if value in [[], {}, None]:
                delete.append(key)
            elif isinstance(value, list):
                dictionary[key] = [item.getJson(True) for item in value]
            elif isinstance(value, Enumerations.Enum):
                dictionary[key] = value.value
            elif isinstance(value, (datetime.date, datetime.datetime)):
                dictionary[key] = Dates.getDateOrDateTimeString(value)
            elif isinstance(value, Variant.Variant):
                dictionary[key] = value.getJson()
            else:
                dictionary[key] = value
        for key in delete:
            del dictionary[key]
        return dictionary

    def getJson(self):
        # Get all attributes as a dictionary
        data = self.__dict__.copy()
        # Delete object private prefix from attribute names
        oldKeys = list(data.keys())
        for key in oldKeys:
            newKey = key.replace('_' + self.__class__.__name__ + '__', '')
            newKey = newKey.replace('_' + self.getBase() + '__', '')
            data[newKey] = data.pop(key)
        # Serialize object within the attribute and delete empty attributes
        data = self.__serializeAndClean(data)
        # jsonize data
        dataJson = json.dumps(data)
        return dataJson




import typing as typ
import datetime as dt

multiDate = typ.Union[dt.date, dt.datetime]


class DateFormatterEn:
    _INT_TO_WEEKDAY_EN: typ.List[str] = ["Monday", "Tuesday", "Wednesday",
                                         "Thursday", "Friday", "Saturday",
                                         "Sunday"]
    _INT_TO_MONTH_EN: typ.List[str] = ["January", "Febuary", "March",
                                       "April", "May", "June",
                                       "July", "August", "September",
                                       "October", "November", "December"]
    _SUFFIX_TO_INDEX: typ.Dict[str, typ.Set[int]] = {"st": {1},
                                                     "nd": {2},
                                                     "rd": {3},
                                                     "th": set(range(4, 10)) | {0}
                                                     }

    @classmethod
    def getLongDateStringEn(cls, date) -> str:
        """

        Args:
            date:

        Returns: input date in long written format.

        """
        weekdayString = cls._INT_TO_WEEKDAY_EN[date.weekday()]
        monthString = cls._INT_TO_MONTH_EN[date.month - 1]
        return f"{weekdayString}, {monthString} {date.day}{cls._getSuffixEn(date)}, {date.year}"

    @classmethod
    def _getSuffixEn(cls, date: multiDate):
        """

        Args:
            date:

        Returns: english language suffix for the day of the date. ex: 31-9-2022 -> "th"

        """
        lastDigit = int(str(date.day)[-1])
        for suffix, intSet in cls._SUFFIX_TO_INDEX.items():
            if lastDigit in intSet:
                return suffix
        raise ValueError(f"Invalid date argument: {date}")


if __name__ == '__main__':
    t = dt.date(2022, 3, 30)
    print(DateFormatterEn.getLongDateStringEn(t))



import bokeh.io as bio
import bokeh.models as bm
import pandas as pd
import typing as typ

from pacifico_devel.util.reportTemplate.src.core.Plot.Table.Table import Table


class TableDefault(Table):

    def makePlot(self,
                 df: pd.DataFrame,
                 title: str,
                 backend: str,
                 shape: typ.Tuple[int, int]) -> bm.DataTable:
        """
        Renders the plot both the HTML and saves the .png.

        Args:
            df:
            title:
            backend:
            shape:

        Returns:

        """
        columns = [bm.TableColumn(field=col, title=col)
                   for col in df.columns]
        source = bm.ColumnDataSource(df)
        dataTable = bm.DataTable(source=source,
                                 columns=columns,
                                 fit_columns=True,
                                 selectable=True,
                                 sortable=True)
        dataTable.name = title

        self._saveImageFile(dataTable)
        return dataTable


if __name__ == '__main__':
    from pacifico_devel.util.reportTemplate.src.util.testing import makeRandomCategoricalDf

    df = makeRandomCategoricalDf(shape=(20, 3))

    bp = TableDefault()
    ps = bp.makePlot(df=df)
    bio.show(ps)



import pacifico_devel.util.reportTemplate.src.core.Plot.PlotLine as pltl
import pacifico_devel.util.reportTemplate.src.core.Plot.PlotBar as pltb
import pacifico_devel.util.reportTemplate.src.core.Plot.Table.TableDataframeImage as pltt
import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm

import pacifico_devel.util.lexikon.src.util.Enumerations as lenm


class PlotFactory:
    @staticmethod
    def getPlot(plotKind: renm.PlotKind,
                languageInput: lenm.Language,
                languageOutput: lenm.Language):

        kwargs = {"languageInput": languageInput,
                  "languageOutput": languageOutput}

        if plotKind is renm.PlotKind.LINE_PLOT:
            return pltl.PlotLine(**kwargs)
        elif plotKind is renm.PlotKind.BAR_PLOT:
            return pltb.PlotBar(**kwargs)
        elif plotKind is renm.PlotKind.TABLE_PLOT:
            return pltt.TableDataframeImage(**kwargs)

        raise ValueError(f"Invalid plot kind: {plotKind}")



import pathlib as pth
import typing as typ
import pandas as pd
import datetime as dt

from pacifico_devel.util.dropbox import Dropbox as dbx
import pacifico_devel.util.reportTemplate.executables.CONFIG as CFG

multiPath = typ.Union[str, pth.Path]


def _isDateLike(path: pth.Path) -> bool:
    return all(s.isnumeric() for s in path.stem.split("-"))


def _isExcelLike(path: pth.Path) -> bool:
    return path.suffix == ".xlsx"


def _getDate(stem: str):
    return dt.datetime.strptime(stem, CFG.DATETIME_FORMAT)


def _readExcelDbx(path: multiPath,
                  sheetName: str,
                  indexCol: typ.Union[int, str]) -> pd.DataFrame:
    dbx.downloadFile(dropboxPath=path)
    df = pd.read_excel(path.name,
                       sheet_name=sheetName,
                       index_col=indexCol)
    pth.Path(path.name).unlink()
    return df


####################

def getDataProyecciones(sheetName: str = "Mayores_Incidencias",
                        indexCol: typ.Optional[int] = None) -> pd.DataFrame:
    """

    Args:
        sheetName:
        indexCol:

    Returns: most recent excel in 'proyecciones' folder.

    """
    items = dbx.listPath(CFG.ROOT_DROPBOX_PROYECCIONES)
    filePaths = [pth.Path(item.path_display)
                 for item in items]
    filePaths: typ.List[pth.Path] = [filePath
                                     for filePath in filePaths
                                     if _isExcelLike(filePath) and _isDateLike(filePath)]
    maxPath: pth.Path = max(filePaths,
                            key=lambda p: _getDate(p.stem))
    return _readExcelDbx(path=maxPath, sheetName=sheetName, indexCol=indexCol)


if __name__ == '__main__':
    # items = dbx.listPath("Pacifico/Chile/IPC/Proyecciones")
    # print(items)
    print(getDataProyecciones())



import pandas as pd
import psycopg2 as pcg
import typing as typ

import pacifico_devel.util.cfg.Configuration as cfg
import pacifico_devel.util.reportTemplate.src.util.dataSource.dataSource as ds
import pacifico_devel.util.databaselegacy.CONFIG as CFG
import pacifico_devel.util.databaselegacy.Enumerations as enm


class PacificoDb(ds.DataSource):
    _host = CFG.HOST
    _port = CFG.PORT
    _user = CFG.USER
    _password = CFG.PASSWORD

    @classmethod
    def makeQuery(cls,
                  sqlQuery: str,
                  database: typ.Union[enm.DataBase, str] = enm.DataBase.Tasas,
                  toDateTime: bool = True) -> pd.DataFrame:
        """
        Returns data from database in dataFrame form.

        Returns (pd.DataFrame): data from database.
        """

        if any(s.lower() in CFG.SQL_MODIFICATION_WORDS for s in sqlQuery.split(" ")):
            raise ValueError("Can only make requests to server.")

        if isinstance(database, str):
            database = enm.DataBase[database]
        database = str(database)
        conn = pcg.connect(user=cls._user,
                           password=cls._password,
                           host=cls._host,
                           port=cls._port,
                           database=database)

        print(f"Loading from database {database}...")
        cursor = conn.cursor()
        cursor.execute(sqlQuery)
        data = cursor.fetchall()
        columns = [col[0] for col in cursor.description]
        conn.close()
        print(f"Loading from database {database}... DONE!")

        df = pd.DataFrame(data, columns=columns)

        if toDateTime:
            if "date" in df.columns:
                df["date"] = pd.to_datetime(df.date)
        return df

    @classmethod
    def mostRecentEntryInDb(cls,
                            ticker: str):
        """

        Returns: most recent dates asociated to a given ticker, if they exist on any table of the database.

        """
        query = "SELECT * FROM information_schema.tables"
        dfTables = cls.makeQuery(sqlQuery=query)
        return dfTables


if __name__ == '__main__':
    print(cfg.get("Token_Pacifico"))
    print()



"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

import datetime
from pacifico.core.Service.ServiceRequest.ServiceRequestDataPoint import ServiceRequestDataPoint
from pacifico.core.Service.Field import FieldVersion, FieldReport
from pacifico.core.Service.Field import FieldDataType, FieldQuality, FieldTimeInterval
from pacifico.util import Variant
from pacifico.util import Enumerations


class ServiceRequestDataPointReport(ServiceRequestDataPoint.ServiceRequestDataPoint):
    def __init__(self, reports=[], timeHorizon=[], dataTypes=[], version=[], quality=[], optional={}):
        ServiceRequestDataPoint.ServiceRequest.Service.Object.__init__(self)
        ServiceRequestDataPoint.ServiceRequestDataPoint.__init__(self, reports, timeHorizon, dataTypes, version, quality, optional)

    @staticmethod
    def create(document='', item='', chapter='', section='', subsection='', paragraph='', dateStart=datetime.datetime.today().date(),
               dateEnd=datetime.datetime.today().date(), fixing=Enumerations.Fixing.EOD,
               dataType=Enumerations.FieldType.Field_Unspecified,
               versionType=Enumerations.VersionType.Version_Unspecified, author='',
               version='', quality=Enumerations.Quality.Quality_Unspecified):
        if document == '' and item == '':
            return None
            #raise Exception("The user must specify either the document or the item when requesting a report.")
        variant = Variant.Variant.getEmptyVariant()
        fieldReport = FieldReport.FieldReport(dateStart, fixing, '', '', document, item, chapter, section, subsection, paragraph, variant)
        fieldTimeInterval = FieldTimeInterval.FieldTimeInterval(dateStart, dateEnd, fixing)
        fieldDataType = FieldDataType.FieldDataType(dataType)
        fieldVersion = FieldVersion.FieldVersion(versionType, version, author)
        fieldQuality = FieldQuality.FieldQuality(quality)
        service = ServiceRequestDataPointReport([fieldReport], [fieldTimeInterval], [fieldDataType], [fieldVersion], [fieldQuality])
        return service



import bokeh.io as bio
import bokeh.plotting as bpl
# from bokeh.models import ColumnDataSource, DataTable, TableColumn
import bokeh.themes as bt
import pandas as pd
import typing as typ
import altair as alt

from pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors import PacificoPlotColors
from pacifico_devel.util.reportTemplate.src.core.Plot.Plot import Plot

import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm


class PlotBar(Plot):

    def _makePlotBokeh(self,
                       df: pd.DataFrame,
                       title: str,
                       shape: typ.Tuple[int, int]):

        categoriesColumn = df.columns[0]
        valuesColumn = df.columns[1]

        df["barchart_color"] = df[valuesColumn].apply(self._colorMap)

        ps = bpl.figure(width=shape[0],
                        height=shape[1],
                        y_range=df.sort_values(valuesColumn,
                                               ascending=True)[categoriesColumn],
                        toolbar_location=None)
        bpl.curdoc().theme = bt.Theme(filename=self._BOKEH_THEME_PATH)
        ps.title = title

        ps.hbar(y=categoriesColumn,
                left=0,
                right=valuesColumn,
                height=0.5,
                color="barchart_color",
                source=df)

        self._saveImageFile(ps)

        return ps

    def _makePlotAltair(self,
                        df: pd.DataFrame,
                        title: str,
                        shape: typ.Tuple[int, int]) -> bpl.Figure:

        categoriesColumn = df.columns[0]
        valuesColumn = df.columns[1]

        # colorScale = alt.Scale(domain=df[valuesColumn],
        #                        range=df[valuesColumn].apply(self._colorMap)
        #                        )
        df["barchart_color"] = df[valuesColumn].apply(self._colorMap)

        ps = alt.Chart(df).mark_bar().encode(x=valuesColumn,
                                             y=alt.Y(categoriesColumn, sort="-x"),
                                             color=alt.Color("barchart_color:N", scale=None))
        ps = ps.properties(title=title,
                           width=shape[0],
                           height=shape[1])

        self._saveImageFile(ps)
        return ps

    def makePlot(self,
                 df: pd.DataFrame,
                 title: str,
                 shape: typ.Tuple[int, int],
                 backend: typ.Union[str, renm.Backend] = "bokeh") -> bpl.Figure:

        dfAux = self._translateDataFrame(df)

        assert (isinstance(dfAux, pd.DataFrame))
        assert (dfAux.shape[1] <= 2)

        if dfAux.shape[1] == 1:
            dfAux = dfAux.reset_index()
        if isinstance(backend, str):
            backend = renm.Backend[backend.upper()]

        if backend is renm.Backend.BOKEH:
            return self._makePlotBokeh(df=dfAux,
                                       title=title,
                                       shape=shape)
        elif backend is renm.Backend.ALTAIR:
            return self._makePlotAltair(df=dfAux,
                                        title=title,
                                        shape=shape)
        raise NotImplementedError(f"Backend {backend.name} not implemented.")

    @staticmethod
    def _colorMap(x: float):
        if x > 0:
            return PacificoPlotColors.BLUE.getCodeString()
        return PacificoPlotColors.PINK.getCodeString()


# def plot(self, series: pd.Series):
#     fig = plt.figure(title="this",
#                      x_axis_label="x axis",
#                      y_axis_label="y axis")
#     colors = None
#     # plot.bar(df.column, s: pd.Series)
#     for k, v in zip(series.index, series):
#         color = PacificoColors.getColor[]
#         fig.bar(k, v, legend_label=k, color=color)
#     formatAxesTickers(fig)
#     makeLegendStyle(fig)
#     style(plot)  # not colors or legends
#     hoverTools(plot)
#     return plot

if __name__ == '__main__':
    from pacifico_devel.util.reportTemplate.src.util.testing import makeRandomCategoricalDf

    df = makeRandomCategoricalDf(shape=(20, 3))

    df.rename(columns={"value_0": "value"}, inplace=True)

    bp = PlotBar()
    ps = bp.makePlot(df=df,
                     title="",
                     shape=(1 / 2, 1))
    bio.show(ps)
    # bp.deleteImageFile()



"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager
import os
import shutil
import time
import boto3
import platform
from ..aws import S3
from ..cfg import Configuration

def __enable_download_in_headless_chrome(driver, downloadDirectory):
    """
    This function was pulled from
    https://github.com/shawnbutton/PythonHeadlessChrome/blob/master/driver_builder.py#L44
    There is currently a "feature" in chrome where
    headless does not allow file download: https://bugs.chromium.org/p/chromium/issues/detail?id=696481
    Specifically this comment ( https://bugs.chromium.org/p/chromium/issues/detail?id=696481#c157 )
    saved the day by highlighting that download wasn't working because it was opening up in another tab.
    This method is a hacky work-around until the official chromedriver support for this.
    Requires chrome version 62.0.3196.0 or above.
    """
    driver.execute_script(
        "var x = document.getElementsByTagName('a'); var i; for (i = 0; i < x.length; i++) { x[i].target = '_self'; }")
    # add missing support for chrome "send_command"  to selenium webdriver
    driver.command_executor._commands["send_command"] = ("POST", '/session/$sessionId/chromium/send_command')
    params = {'cmd': 'Page.setDownloadBehavior',
              'params': {'behavior': 'allow', 'downloadPath': downloadDirectory}}  # 'util/selenium'
    command_result = driver.execute("send_command", params)
    print("response from browser:")
    for key in command_result:
        print("result:" + key + ":" + str(command_result[key]))

def _init_bin(executable_name):
    BIN_DIR = "/tmp/bin"
    CURR_BIN_DIR = "/opt/python/bin"
    currfile = os.path.join(CURR_BIN_DIR, executable_name)
    newfile = os.path.join(BIN_DIR, executable_name)
    if not os.path.isfile(newfile):
        start = time.time()
        if not os.path.exists(BIN_DIR):
            print("Creating bin folder")
            os.makedirs(BIN_DIR)
        print("Copying binaries for " + executable_name + " in /tmp/bin")
        shutil.copy2(currfile, newfile)
        print("Giving new binaries permissions for lambda")
        os.chmod(newfile, 0o775)
        elapsed = time.time() - start
        print(executable_name + " ready in " + str(elapsed) + "s.")

def _init_bin_s3(executable_name):
    BIN_DIR = "/tmp/bin"
    s3Filepath = 'lambda_layers/bin/'
    objectFilepath = os.path.join(s3Filepath, executable_name)
    newfile = os.path.join(BIN_DIR, executable_name)
    if not os.path.isfile(newfile):
        if not os.path.exists(BIN_DIR):
            print("Creating bin folder")
            os.makedirs(BIN_DIR)
        client = boto3.client('s3')
        client.download_file('bfi-cfg', objectFilepath, newfile)
        print('{} downloaded!'.format(executable_name))
        print("Giving new binaries permissions for lambda")
        os.chmod(newfile, 0o775)

def getDriverLinux(downloadDirectory=os.getcwd(), detach=False):
    ## Slowness:
    # https://github.com/codeceptjs/CodeceptJS/issues/561
    # https://stackoverflow.com/questions/49032878/disable-proxy-via-python-for-selenium-with-headless-chrome
    # https://stackoverflow.com/questions/62898801/selenium-headless-chrome-runs-much-slower
    ##
    # _init_bin_s3("headless-chromium")
    # _init_bin_s3("chromedriver")
    # Permissions
    # os.chmod("util/selenium/bin/chromedriver", 0o775)
    # os.chmod("util/selenium/bin/headless-chromium", 0o775)
    chrome_options = webdriver.ChromeOptions()
    if detach:
        chrome_options.add_experimental_option("detach", True)
    chrome_options.add_argument('--headless')
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument('--disable-gpu')
    chrome_options.add_argument('--window-size=1280,720')
    chrome_options.add_argument("--start-maximized")
    # chrome_options.add_argument("enable-automation")
    # chrome_options.add_argument("--disable-infobars")
    chrome_options.add_argument("--disable-extensions")
    # chrome_options.add_argument('--user-data-dir={}'.format(downloadDirectory))
    # chrome_options.add_argument('--hide-scrollbars')
    # chrome_options.add_argument('--enable-logging')
    # chrome_options.add_argument('--log-level=0')
    # chrome_options.add_argument('--v=99')
    # chrome_options.add_argument('--single-process')
    # chrome_options.add_argument('--data-path={}'.format(downloadDirectory))
    # chrome_options.add_argument('--ignore-certificate-errors')
    # chrome_options.add_argument('--homedir={}'.format(downloadDirectory))
    chrome_options.add_argument('--disable-dev-shm-usage')
    # chrome_options.add_argument('--disk-cache-dir={}'.format(downloadDirectory))
    chrome_options.add_argument('user-agent=Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36')
    chrome_options.add_argument('--no-proxy-server')
    chrome_options.add_argument("--proxy-server='direct://'")
    chrome_options.add_argument("--proxy-bypass-list=*")
    chrome_options.add_argument("--proxy-server=")
    chrome_options.add_argument("--dns-prefetch-disable")
    chrome_options.add_argument("--enable-features=NetworkService,NetworkServiceInProcess")
    # chrome_options.add_argument('blink-settings=imagesEnabled=false')
    # chrome_options.binary_location = "/usr/bin/chromium-browser"
    # don't tell chrome that it is automated
    # chrome_options.add_experimental_option("excludeSwitches", ["enable-automation"])
    # chrome_options.add_experimental_option('useAutomationExtension', False)
    prefs = {'download.default_directory': downloadDirectory,
             'download.prompt_for_download': False,
             'download.directory_upgrade': True,
             'safebrowsing.enabled': False,
             'safebrowsing.disable_download_protection': True,
             'profile.default_content_setting_values.automatic_downloads': 1} #,
             # 'profile.managed_default_content_settings.images': 2}
    chrome_options.add_experimental_option('prefs', prefs)
    driver = webdriver.Chrome(executable_path=ChromeDriverManager().install(), chrome_options=chrome_options)  # , executable_path="util/selenium/bin/chromedriver", service_log_path='util/selenium/chromedriver.log')
    # __enable_download_in_headless_chrome(driver, downloadDirectory)
    return driver

def getDriverWindows(downloadDirectory=os.getcwd(), detach=False):
    chrome_options = webdriver.ChromeOptions()
    if detach:
        chrome_options.add_experimental_option("detach", True)
    # chrome_options.add_argument("enable-automation")
    prefs = {'download.default_directory': downloadDirectory,
             'download.prompt_for_download': False,
             'download.directory_upgrade': True,
             'safebrowsing.enabled': False,
             'safebrowsing.disable_download_protection': True,
             'profile.default_content_setting_values.automatic_downloads': 1}
    chrome_options.add_experimental_option('prefs', prefs)
    # pacificoBots = 'Pacifico-Bots'
    # chromedriverPath = os.getcwd().replace("\\", '/').split(pacificoBots)[0] + pacificoBots + '/pacifico_devel/util/selenium/binWindows/chromedriver'
    driver = webdriver.Chrome(executable_path=ChromeDriverManager().install(), chrome_options=chrome_options)
    return driver

def getDriverMac(downloadDirectory=os.getcwd(), detach=False):
    chrome_options = webdriver.ChromeOptions()
    if detach:
        chrome_options.add_experimental_option("detach", True)
    prefs = {'download.default_directory': downloadDirectory,
             'download.prompt_for_download': False,
             'download.directory_upgrade': True,
             'safebrowsing.enabled': False,
             'safebrowsing.disable_download_protection': True,
             'profile.default_content_setting_values.automatic_downloads': 1}
    chrome_options.add_experimental_option('prefs', prefs)
    return webdriver.Chrome(ChromeDriverManager().install(), chrome_options=chrome_options)

def getDriverLinuxChromium(downloadDirectory, detach=False):
    chromedriverPath = r'/usr/bin/chromedriver_legacy'  # os.getcwd().replace("\\", '/') + '/pacifico_devel/util/selenium/binLinux/chromedriver'
    # os.chmod(chromedriverPath, 0o775)
    headlessChromiumPath = r'/usr/bin/headless-chromium'  # os.getcwd().replace("\\", '/') + '/pacifico_devel/util/selenium/binLinux/headless-chromium'
    # os.chmod(headlessChromiumPath, 0o775)
    # _init_bin_s3("headless-chromium")
    # _init_bin_s3("chromedriver")
    chrome_options = webdriver.ChromeOptions()
    if detach:
        chrome_options.add_experimental_option("detach", True)
    chrome_options.add_argument('--headless')
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument('--disable-gpu')
    chrome_options.add_argument('--window-size=1280x1696')
    chrome_options.add_argument('--user-data-dir={}'.format(downloadDirectory))
    chrome_options.add_argument('--hide-scrollbars')
    chrome_options.add_argument('--enable-logging')
    chrome_options.add_argument('--log-level=0')
    chrome_options.add_argument('--v=99')
    chrome_options.add_argument('--single-process')
    chrome_options.add_argument('--data-path={}'.format(downloadDirectory))
    chrome_options.add_argument('--ignore-certificate-errors')
    chrome_options.add_argument('--homedir={}'.format(downloadDirectory))
    chrome_options.add_argument('--disable-dev-shm-usage')
    chrome_options.add_argument('--disk-cache-dir={}'.format(downloadDirectory))
    chrome_options.add_argument('user-agent=Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36')
    chrome_options.binary_location = headlessChromiumPath
    prefs = {'download.default_directory': downloadDirectory,
             'download.prompt_for_download': False,
             'download.directory_upgrade': True,
             'safebrowsing.enabled': False,
             'safebrowsing.disable_download_protection': True,
             'profile.default_content_setting_values.automatic_downloads': 1}
    chrome_options.add_experimental_option('prefs', prefs)
    driver = webdriver.Chrome(chromedriverPath, chrome_options=chrome_options, service_log_path=downloadDirectory)
    __enable_download_in_headless_chrome(driver)
    return driver

def getDriver(downloadDirectory=os.getcwd(), chromium=False, detach=False):
    try:
        if chromium:
            return getDriverLinuxChromium(downloadDirectory, detach)
        elif 'mac' in platform.platform():
            return getDriverMac(downloadDirectory, detach)
        elif 'Linux' in platform.platform():
            return getDriverLinux(downloadDirectory, detach)
        else:
            return getDriverWindows(downloadDirectory, detach)
    except Exception as e:
        import traceback
        import sys
        traceback.print_exc()
        exc_type, exc_value, exc_traceback = sys.exc_info()
        lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
        x = ''.join('!! ' + line for line in lines)
        awsAccessKey = Configuration.get('AWS_ACCESS_KEY')
        awsSecretKey = Configuration.get('AWS_SECRET_KEY')
        S3.writeToBucket(str(e) + str(x), "selenium.txt", awsAccessKey=awsAccessKey, awsSecretKey=awsSecretKey)




"""Copyright © 2020 Burrus Financial Intelligence, Ltda. (hereafter, BFI) Permission to include in application
software or to make digital or hard copies of part or all of this work is subject to the following licensing
agreement.
BFI Software License Agreement: Any User wishing to make a commercial use of the Software must contact BFI
at jacques.burrus@bfi.lat to arrange an appropriate license. Commercial use includes (1) integrating or incorporating
all or part of the source code into a product for sale or license by, or on behalf of, User to third parties,
or (2) distribution of the binary or source code to third parties for use with a commercial product sold or licensed
by, or on behalf of, User. """

__all__ = ['Country', 'VersionType', 'Quality', 'Fixing']

from enum import Enum
import datetime
from pacifico.util import Dates


class Fixing(Enum):
    EOD = -1
    F_0000 = 0
    F_0030 = 30
    F_0100 = 100
    F_0130 = 130
    F_0200 = 200
    F_0230 = 230
    F_0300 = 300
    F_0330 = 330
    F_0400 = 400
    F_0430 = 430
    F_0500 = 500
    F_0530 = 530
    F_0600 = 600
    F_0630 = 630
    F_0700 = 700
    F_0730 = 730
    F_0800 = 800
    F_0830 = 830
    F_0900 = 900
    F_0930 = 930
    F_1000 = 1000
    F_1030 = 1030
    F_1100 = 1100
    F_1130 = 1130
    F_1200 = 1200
    F_1230 = 1230
    F_1300 = 1300
    F_1330 = 1330
    F_1400 = 1400
    F_1430 = 1430
    F_1500 = 1500
    F_1530 = 1530
    F_1600 = 1600
    F_1630 = 1630
    F_1700 = 1700
    F_1730 = 1730
    F_1800 = 1800
    F_1830 = 1830
    F_1900 = 1900
    F_1930 = 1930
    F_2000 = 2000
    F_2030 = 2030
    F_2100 = 2100
    F_2130 = 2130
    F_2200 = 2200
    F_2230 = 2230
    F_2300 = 2300
    F_2330 = 2330

    @staticmethod
    def getTime(fixing):
        if not isinstance(fixing, Fixing):
            raise TypeError("The fixing should be set to a class 'Enumerations.Fixing'.")
        if fixing == Fixing.EOD:
            hour = 17
            minute = 0
            second = 0
        else:
            hour = int(fixing.name[2]) * 10 + int(fixing.name[3])
            minute = int(fixing.name[4]) * 10 + int(fixing.name[5])
            second = 0
        return hour, minute, second

    @staticmethod
    def getString(fixing):
        return fixing.name

class Country(Enum):
    Country_RiskFactors = -1
    Country_Unspecified = 0
    Country_Chile = 1
    Country_Colombia = 2
    Country_Peru = 3
    Country_Mexico = 4
    Country_Argentina = 5
    Country_United_States = 6
    Country_Europe = 7
    Country_Japan = 8
    Country_International = 999

    @staticmethod
    def fromString(string):
        if string == 'Chile':
            return Country.Country_Chile
        elif string == 'Colombia':
            return Country.Country_Colombia
        elif string == 'Peru':
            return Country.Country_Peru
        elif string == 'Mexico':
            return Country.Country_Mexico
        elif string == 'Argentina':
            return Country.Country_Argentina
        elif string == 'United States':
            return Country.Country_United_States
        elif string == 'Europe':
            return Country.Country_Europe
        elif string == 'Japan':
            return Country.Country_Japan
        elif string == 'International':
            return Country.Country_International
        elif string == 'Risk Factors' or string == 'RiskFactors':
            return Country.Country_RiskFactors
        else:
            return Country.Country_Unspecified

    @staticmethod
    def getString(country):
        countryString = country.name.split('_')[1:]
        returnString = ''
        for string in countryString:
            returnString += string + ' '
        return returnString[:-1]

class FieldType(Enum):
    Field_Unspecified = 0
    Field_Price = 1
    Field_Yield = 2
    Field_Duration = 3
    Field_Convexity = 4
    Field_Delta = 5
    Field_Gamma = 6
    Field_Vega = 7
    Field_Volatility = 8
    Field_Quote = 9
    Field_CleanPrice = 10
    Field_CleanQuote = 11
    Field_MarketPresence = 12

    @staticmethod
    def getString(fieldType):
        fieldTypeString = fieldType.name.split('_')[1]
        if 'Clean' in fieldTypeString:
            fieldTypeString = fieldTypeString.replace('Clean', 'Clean ')
        elif 'Market' in fieldTypeString:
            fieldTypeString = fieldTypeString.replace('Market', 'Market ')
        return fieldTypeString

    @staticmethod
    def fromString(fieldTypeString):
        fieldTypeString = fieldTypeString.lower()
        if fieldTypeString == 'unspecified':
            return FieldType.Field_Unspecified
        elif fieldTypeString == 'price':
            return FieldType.Field_Price
        elif fieldTypeString == 'yield':
            return FieldType.Field_Yield
        elif fieldTypeString == 'duration':
            return FieldType.Field_Duration
        elif fieldTypeString == 'convexity':
            return FieldType.Field_Convexity
        elif fieldTypeString == 'delta':
            return FieldType.Field_Delta
        elif fieldTypeString == 'gamma':
            return FieldType.Field_Gamma
        elif fieldTypeString == 'vega':
            return FieldType.Field_Vega
        elif fieldTypeString == 'volatility':
            return FieldType.Field_Volatility
        elif fieldTypeString == 'quote':
            return FieldType.Field_Quote
        elif fieldTypeString == 'clean price':
            return FieldType.Field_CleanPrice
        elif fieldTypeString == 'clean quote':
            return FieldType.Field_CleanQuote
        elif fieldTypeString == 'market presence':
            return FieldType.Field_MarketPresence
        else:
            return FieldType.Field_Unspecified

    @staticmethod
    def fromValueOrString(valueOrString):
        if isinstance(valueOrString, int):
            return FieldType(valueOrString)
        elif isinstance(valueOrString, str):
            return FieldType.fromString(valueOrString)
        else:
            return None

class Quality(Enum):
    Quality_Unspecified = 0
    Quality_Production = 1
    Quality_Certification = 2
    Quality_Development = 3

    @staticmethod
    def getString(quality):
        qualityString = quality.name.split('_')[1]
        return qualityString

    @staticmethod
    def fromString(qualityString):
        if qualityString == 'Unspecified':
            return Quality.Quality_Unspecified
        elif qualityString == 'Production':
            return Quality.Quality_Production
        elif qualityString == 'Certification':
            return Quality.Quality_Certification
        elif qualityString == 'Development':
            return Quality.Quality_Development
        else:
            return Quality.Quality_Unspecified

class VersionType(Enum):
    Version_Pricing = 1
    Version_Prediction = 2
    Version_Unspecified = 0

    @staticmethod
    def getString(versionType):
        versionTypeString = versionType.name.split('_')[1]
        return versionTypeString

    @staticmethod
    def fromString(versionTypeString):
        if versionTypeString == 'Pricing':
            return VersionType.Version_Pricing
        elif versionTypeString == 'Prediction':
            return VersionType.Version_Prediction
        elif versionTypeString == 'Unspecified':
            return VersionType.Version_Unspecified
        else:
            return VersionType.Version_Unspecified

class ObjectType(Enum):
    Identification = 100
    IdentificationName = 110
    IdentificationFamily = 111
    IdentificationGroup = 112
    IdentificationMarket = 113
    IdentificationMnemo = 114
    IdentificationChileRut = 120
    IdentificationColombiaNit = 121
    IdentificationMexicoRfc = 122
    IdentificationPeruRuc = 123
    IdentificationTicker = 130
    Agent = 200
    AgentUser = 210
    DataPoint = 300
    DataPointComment = 310
    DataPointTicker = 320
    Version = 400
    Authentification = 500
    AuthentificationAWSAPIKey = 510

class PeruRucType(Enum):
    PERU_RUC_TYPE_PERSONA_NATURAL = 10
    PERU_RUC_TYPE_PERSONA_JURIDICA = 20
    PERU_RUC_TYPE_OTROS = 15 # sucesiones indivisas, sociedades conyugales, C FFAA, C FFPP, DPI, Carnet de Extranjeria
    PERU_RUC_TYPE_VALIDO = 16
    PERU_RUC_TYPE_INSCRIPCION_ENTRE_1993_Y_2000 = 17

class UsagePlan(Enum):
    UsagePlan_Client = 0
    UsagePlan_Author = 1
    UsagePlan_Administrator = 2

class Action(Enum):
    Action_Enable = 1
    Action_Disable = -1

class Providers(Enum):
    Provider_BFI = 0
    Provider_AlphaVantage = 1

class Format(Enum):
    Json = 1
    Dictionary = 2
    DataFrame = 3

    @staticmethod
    def fromString(formatString):
        if formatString == 'json' or formatString == 'JSON':
            return Format.Json
        elif formatString == 'dictionary' or formatString == 'dict':
            return Format.Dictionary
        elif formatString == 'dataFrame' or formatString == 'DataFrame' or formatString == 'df':
            return Format.DataFrame
        else:
            return None

class ValueType(Enum):
    ValueType_Bool = 0
    ValueType_Integer = 1
    ValueType_Double = 2
    ValueType_String = 3
    ValueType_Date = 4
    ValueType_DateTime = 5
    ValueType_Url = 6
    ValueType_File = 7
    ValueType_Browser = 8

    def getString(self):
        return self.name.split('_')[1]

    @staticmethod
    def fromString(valueTypeString):
        for valueTypeEnum in ValueType:
            if valueTypeString == valueTypeEnum.getString():
                return valueTypeEnum

    def forceType(self, value):
        if self == ValueType.ValueType_Bool:
            return bool(value)
        elif self == ValueType.ValueType_Integer:
            return int(value)
        elif self == ValueType.ValueType_Double:
            return float(value)
        elif self == ValueType.ValueType_String:
            return str(value)
        elif self == ValueType.ValueType_Date:
            if isinstance(value, str):
                return Dates.dateFromString(value)
            return Dates.julianToDate(float(value))
        elif self == ValueType.ValueType_DateTime:
            if isinstance(value, str):
                return Dates.dateTimeFromString(value)
            return Dates.julianToDateTime(float(value))
        elif self == ValueType.ValueType_Url or self == ValueType.ValueType_Browser:
            return str(value)
        elif self == ValueType.ValueType_File:
            return str(value)
        else:
            return None

    def fromDB(self, value):
        if self == ValueType.ValueType_Bool:
            if isinstance(value, (int, float)):
                return bool(int(value))
            if isinstance(value, str):
                if value.isnumeric():
                    return bool(int(value))
                else:
                    if value.lower() in ['false', 'f']:
                        return False
                    else:
                        return True
        elif self == ValueType.ValueType_Integer:
            return int(value)
        elif self == ValueType.ValueType_Double:
            return float(value)
        elif self == ValueType.ValueType_String:
            return str(value)
        elif self == ValueType.ValueType_Date:
            return Dates.getDateString(Dates.julianToDate(float(value)))
        elif self == ValueType.ValueType_DateTime:
            return Dates.getDateTimeString(Dates.julianToDateTime(float(value)))
        elif self == ValueType.ValueType_Url:
            return str(value)
        elif self == ValueType.ValueType_File:
            return str(value)
        else:
            return None

    def getJson(self, value):
        if self == ValueType.ValueType_Bool:
            return bool(value)
        elif self == ValueType.ValueType_Integer:
            return int(value)
        elif self == ValueType.ValueType_Double:
            return float(value)
        elif self == ValueType.ValueType_String:
            return str(value)
        elif self == ValueType.ValueType_Date:
            return Dates.getDateString(Dates.julianToDate(float(value)))
        elif self == ValueType.ValueType_DateTime:
            return Dates.getDateTimeString(Dates.julianToDateTime(float(value)))
        elif self == ValueType.ValueType_Url:
            return str(value)
        elif self == ValueType.ValueType_File:
            return str(value)
        else:
            return None

    def getType(self):
        if self == ValueType.ValueType_Bool:
            return bool
        elif self == ValueType.ValueType_Integer:
            return int
        elif self == ValueType.ValueType_Double:
            return float
        elif self == ValueType.ValueType_String:
            return str
        elif self == ValueType.ValueType_Date:
            return datetime.date
        elif self == ValueType.ValueType_DateTime:
            return datetime.datetime
        elif self == ValueType.ValueType_Url:
            return str
        elif self == ValueType.ValueType_File:
            return str
        else:
            return type(None)

    def fromJson(self, value):
        if self == ValueType.ValueType_Bool:
            return bool(value)
        elif self == ValueType.ValueType_Integer:
            return int(value)
        elif self == ValueType.ValueType_Double:
            return float(value)
        elif self == ValueType.ValueType_String:
            return str(value)
        elif self == ValueType.ValueType_Date:
            return datetime.date.fromisoformat(value)
        elif self == ValueType.ValueType_DateTime:
            return datetime.datetime.fromisoformat(value)
        elif self == ValueType.ValueType_Url:
            return str(value)
        elif self == ValueType.ValueType_File:
            return str(value)
        else:
            return None


if __name__ == '__main__':
    x = ValueType.ValueType_Bool
    y = ValueType.getString(x)
    z = ValueType.fromString(y)
    print(x)
    print(y)
    print(z)



import typing as typ
import deepl

from pacifico_devel.util.lexikon.src.util.Enumerations import Language
from pacifico_devel.util.cfg import Configuration


class Translator:
    """
    Class that implements language translator (probably with deep learning).
    """
    _KEY_NAME = "deepl_translator_key"

    @classmethod
    def translate(cls,
                  text: typ.Union[str, typ.Iterable[str]],
                  languageInput: Language,
                  languageOutput: Language) -> typ.Union[str, typ.List[str]]:
        """

        Args:
            text:
            languageInput:
            languageOutput:

        Returns: translated text

        """

        translator = deepl.Translator(cls._getKey())
        print(f"SOURCE LANG: {languageInput.deeplStringOutput}")
        print(f"TARGET LANG: {languageOutput.deeplStringOutput}")
        if isinstance(text, str):
            if languageInput is languageOutput:
                return text
            result = translator.translate_text(text=text,
                                               source_lang=languageInput.deeplStringInput,
                                               target_lang=languageOutput.deeplStringOutput)
            return result.text
        elif hasattr(text, "__iter__"):
            if languageInput is languageOutput:
                return list(text)
            result = translator.translate_text(text=text,
                                               source_lang=languageInput.deeplStringInput,
                                               target_lang=languageOutput.deeplStringOutput)
            return [res.text for res in result]

        raise ValueError(f"Invalid input: {text} of type {type(text)}. Must be of type str or an iterable of str.")

    @classmethod
    def _getKey(cls):
        """ Returns key from key-keeper"""
        return Configuration.get(cls._KEY_NAME)


if __name__ == '__main__':
    text = """Aumentamos nuestra proyección de IPC de marzo desde 0.94 % hasta 1.08 % sin consecuencias relevantes para el acumulado del año. El seguimiento
    a productos como cigarrillos y gas por red nos llevó a adelantar la variación mensual que esperábamos ocurriera en el segundo trimestre. Para el caso
del precio del gas licuado empiezan a materializarse los riesgos al alza: desde la proyección inicial del mes preveíamos aumentos significativos, pero
el seguimiento de precios nos llevó a proyectar un incremento aún mayor. Todo lo anterior se vio parcialmente compensado por una mayor caída
proyectada para el pasaje en bus interurbano."""

    translation = Translator.translate(text=text,
                                       languageInput=Language.English_US,
                                       languageOutput=Language.Spanish)

    print(f"ORIGINAL TEXT: {text}")
    print(f"TRANSLATED TEXT: {translation}")



import typing as typ
import bokeh.plotting as bpl
import bokeh.themes as bt
import bokeh.models as bm
import pandas as pd
import altair as alt

from pacifico_devel.util.reportTemplate.src.util.PacificoPlotColors import PacificoPlotColors
from pacifico_devel.util.reportTemplate.src.core.Plot.Plot import Plot
import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm

import pacifico_devel.util.lexikon.src.util.Enumerations as lenm


class PlotLine(Plot):
    _DATE_TIME_FORMAT = lenm.Language.English_US.dateFormatString()

    def _makePlotBokeh(self,
                       df: pd.DataFrame,
                       title: str,
                       shape: typ.Tuple[int, int]) -> bpl.Figure:
        ps = bpl.figure(width=shape[0],
                        height=shape[1],
                        toolbar_location=None)
        bpl.curdoc().theme = bt.Theme(filename=self._BOKEH_THEME_PATH)
        ps.title = title
        if pd.api.types.is_datetime64_any_dtype(df.index):
            ps.xaxis[0].formatter = bm.DatetimeTickFormatter(years=self._DATE_TIME_FORMAT,
                                                             months=self._DATE_TIME_FORMAT,
                                                             days=self._DATE_TIME_FORMAT)
        for colName, color in zip(df.columns,
                                  PacificoPlotColors):
            if pd.api.types.is_numeric_dtype(df[colName]):
                data = df[[colName]]
                ps.line(x="index",
                        y=colName,
                        legend_label=colName,
                        color=color.getCodeString(),
                        source=data,
                        line_width=2)
        self._saveImageFile(ps)
        return ps

    def _makePlotAltair(self,
                        df: pd.DataFrame,
                        title: str,
                        shape: typ.Tuple[int, int]) -> bpl.Figure:
        indexName = df.index.name
        data = df[[colName
                   for colName in df.columns
                   if pd.api.types.is_numeric_dtype(df[colName])]]
        data = data.reset_index()

        varName = self._translateString("series")
        valueName = self._translateString("value")

        data = data.melt(id_vars=data.columns[0],
                         var_name=varName,
                         value_name=valueName)
        colorScale = alt.Scale(domain=data[varName].unique(),
                               range=[c.getCodeString()
                                      for c in PacificoPlotColors]
                               )

        # print(f"DATA: {data}")
        ps = alt.Chart(data).mark_line().encode(x=indexName,
                                                y=valueName,
                                                color=alt.Color(varName, scale=colorScale))
        ps = ps.properties(title=title,
                           width=shape[0],
                           height=shape[1])

        # kwargsTitle = {"font": "klartext mono",
        #                "color": PacificoPlotColors.BLUE.getCodeString()}
        # kwargsAxis = {"title" + k.capitalize(): v
        #               for k, v in kwargsTitle.items()}
        #
        # ps = ps.configure_title(**kwargsTitle).configure_axis(**kwargsAxis)

        self._saveImageFile(ps)
        return ps

    def makePlot(self,
                 df: pd.DataFrame,
                 title: str,
                 shape: typ.Tuple[int, int],
                 backend: typ.Union[str, renm.Backend] = "bokeh"
                 ) -> bpl.Figure:

        dfAux = self._translateDataFrame(df)

        if isinstance(backend, str):
            backend = renm.Backend[backend.upper()]

        if backend is renm.Backend.BOKEH:
            return self._makePlotBokeh(df=dfAux,
                                       title=title,
                                       shape=shape)

        elif backend is renm.Backend.ALTAIR:
            return self._makePlotAltair(df=dfAux,
                                        title=title,
                                        shape=shape)

        raise NotImplementedError(f"Backend {backend.name} not implemented.")


if __name__ == '__main__':
    from pacifico_devel.util.reportTemplate.src.util.testing import makeRandomCategoricalDf

    df = makeRandomCategoricalDf(shape=(20, 3))  # .reset_index()

    print(df)

    bp = PlotLine(languageInput=lenm.Language.Spanish,
                  languageOutput=lenm.Language.French)
    ps = bp.makePlot(df=df,
                     backend="altair",
                     title="LINE PLOT",
                     shape=(1000, 1000)
                     )
    # bio.show(ps)
    # bp.deleteImageFile()



# from abc import ABCMeta, abstractmethod
import abc
import pathlib as pth
import bokeh.io as bio
import pandas as pd
import pandas.api.types as pdt
import typing as typ
import dataframe_image as dfi
import altair as alt

import bokeh.plotting as bpl
import bokeh.models as bm
import altair.vegalite.v4.api as altapi
import pandas.io.formats.style as pdst

from pacifico_devel.util.reportTemplate.src.core.Style.Style import Style
import pacifico_devel.util.reportTemplate.src.util.Enumerations as renm

import pacifico_devel.util.lexikon.src.core.Translator as trl
import pacifico_devel.util.lexikon.src.util.Enumerations as lenm

from pacifico_devel.util.reportTemplate.src.util.Patches import ScreenshotPatch

MultiDisplay = typ.Union[bpl.Figure, bm.DataTable, altapi.Chart, pdst.Styler]


class Plot(metaclass=abc.ABCMeta):
    """
    Abstract class for all necessary plots
    """
    _BOKEH_THEME_PATH = pth.Path(__file__).parents[1] / "Style" / "bokeh_themes" / "pacifico_theme.yml"
    _TEMP_ROOT = pth.Path(__file__).parent / "tmp"

    def __init__(self,
                 languageInput: lenm.Language,
                 languageOutput: lenm.Language,
                 style: typ.Optional[Style] = None):
        alt.data_transformers.disable_max_rows()
        self._languageInput = languageInput
        self._languageOutput = languageOutput
        self._style = style
        self._pathImage = self._makePathImage()

    @abc.abstractmethod
    def makePlot(self,
                 df: pd.DataFrame,
                 title: str,
                 backend: typ.Union[str, renm.Backend],
                 shape: typ.Tuple[int, int]) -> MultiDisplay:
        """
        Renders plot and saves it to temp folder.

        Args:
            df:
            title:
            backend:
            shape:

        Returns:

        """
        pass

    @property
    def pathImage(self) -> pth.Path:
        """
        Path in which the image of the plot will be saved.

        Returns:

        """
        return self._pathImage

    def _translateDataFrame(self, df: pd.DataFrame) -> pd.DataFrame:
        """
        Translates all strings in dataframe.

        Args:
            df:

        Returns:

        """
        dfAux = df.copy()
        if pdt.is_string_dtype(dfAux.index):
            dfAux.index = trl.Translator.translate(text=dfAux.index,
                                                   languageInput=self._languageInput,
                                                   languageOutput=self._languageOutput)

        if dfAux.index.name is None:
            dfAux.index.name = trl.Translator.translate(text="index",
                                                        languageInput=lenm.Language.English_US,
                                                        languageOutput=self._languageOutput)
        else:
            dfAux.index.name = trl.Translator.translate(text=dfAux.index.name,
                                                        languageInput=self._languageInput,
                                                        languageOutput=self._languageOutput)

        if pdt.is_string_dtype(dfAux.columns):
            dfAux.columns = trl.Translator.translate(text=dfAux.columns,
                                                     languageInput=self._languageInput,
                                                     languageOutput=self._languageOutput)
        for col in dfAux.columns:
            if pdt.is_string_dtype(dfAux[col]):
                dfAux[col] = trl.Translator.translate(text=dfAux[col],
                                                      languageInput=self._languageInput,
                                                      languageOutput=self._languageOutput)
        return dfAux

    def _translateString(self, s: str) -> str:
        """
        Translates a single string.

        Args:
            s:

        Returns:

        """
        return trl.Translator.translate(text=s,
                                        languageInput=self._languageInput,
                                        languageOutput=self._languageOutput)

    def deleteImageFile(self) -> None:
        """
        Deletes plot image file.

        Returns:

        """
        self._pathImage.unlink()

    def hoverTools(self):
        """
        Hover tools for plot HTML.

        Returns:

        """
        pass

    def _saveImageFile(self,
                       Display: MultiDisplay) -> None:
        """
        Saves Image of plot to 'tmp' folder.

        Args:
            Display:

        Returns:

        """
        if isinstance(Display, (bpl.Figure, bm.DataTable)):
            bio.export_png(Display,
                           filename=self._pathImage)
        elif isinstance(Display, altapi.Chart):
            Display.save(fp=self._makePathImage())
        elif isinstance(Display, pdst.Styler):
            ScreenshotPatch.patch()  # Monkey patch for chrome screenshots server
            print(f"SAVING TABLE TO: {self._pathImage} ...")
            dfi.export(obj=Display,
                       filename=str(self._pathImage))
            print(f"SAVING TABLE TO: {self._pathImage} ... DONE!")

        else:
            raise ValueError(f"Display of type {type(Display)} cannot be saved.")

    @classmethod
    def _makePathImage(cls) -> pth.Path:
        idxMax = cls._getMaximumDigit()

        if idxMax is None:
            plotIdx = 0
        else:
            plotIdx = idxMax + 1

        fileName = f"plot_{plotIdx}.png"
        return cls._TEMP_ROOT / fileName

    @classmethod
    def _getMaximumDigit(cls) -> typ.Optional[int]:
        digits = [cls._digitFromFileName(filePath.stem)
                  for filePath in cls._TEMP_ROOT.glob("*.png")]
        digits = [x
                  for x in digits
                  if x is not None]
        if len(digits) > 0:
            return max(digits)
        return None

    @staticmethod
    def _digitFromFileName(s: str) -> typ.Optional[int]:
        base = s.split("_")[-1]
        if base.isnumeric():
            return int(base)
        return None


if __name__ == '__main__':
    print("/n")
    p = pth.Path(__file__).parent / "tmp"
    print(p)
    for x in p.glob("*.png"):
        print(x)
